package com.massa.util.extensions.jdk8;

import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.Month;
import java.time.MonthDay;
import java.time.Year;
import java.time.YearMonth;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;

import javax.xml.datatype.XMLGregorianCalendar;

import org.junit.Assert;
import org.junit.Test;

import com.massa.mrules.accessor.MDateDiffAccessor;
import com.massa.mrules.accessor.MDateManipulationAccessor;
import com.massa.mrules.accessor.MValueAccessor;
import com.massa.mrules.addon.MAddonsFinder;
import com.massa.mrules.context.compile.MCompilationContext;
import com.massa.mrules.context.execute.IExecutionContext;
import com.massa.mrules.context.execute.MExecutionContext;
import com.massa.mrules.extensions.jdk8.addon.Jdk8VariablesManager;
import com.massa.mrules.extensions.jdk8.addon.Jdk8VariablesManager.Jdk8Now;
import com.massa.util.convert.ConvertUtils;
import com.massa.util.extensions.jdk8.convert.Jdk8DateTimeConverter;

public class Jdk8Test {
	@Test
	public void testJdk8Conv() throws Exception {
		final Calendar c = Calendar.getInstance();
		c.set(Calendar.YEAR, 2016);
		c.set(Calendar.MONTH, Calendar.AUGUST);
		c.set(Calendar.DATE, 25);
		c.set(Calendar.HOUR_OF_DAY, 12);
		c.set(Calendar.MINUTE, 34);
		c.set(Calendar.SECOND, 56);
		c.set(Calendar.MILLISECOND, 789);

		final Date d = ConvertUtils.getDefaultInstance().cast(Date.class, c);
		Assert.assertEquals(c.getTime(), d);

		final LocalDate ldC = ConvertUtils.getDefaultInstance().cast(LocalDate.class, c);
		final LocalDate ldD = ConvertUtils.getDefaultInstance().cast(LocalDate.class, d);

		Assert.assertEquals(ldC, ldD);
		Assert.assertEquals(2016, ldC.getYear());
		Assert.assertEquals(8, ldC.getMonthValue());
		Assert.assertEquals(25, ldC.getDayOfMonth());

		final LocalDateTime ldtC = ConvertUtils.getDefaultInstance().cast(LocalDateTime.class, c);
		final LocalDateTime ldtD = ConvertUtils.getDefaultInstance().cast(LocalDateTime.class, d);

		Assert.assertEquals(ldtC, ldtD);
		Assert.assertEquals(2016, ldtC.getYear());
		Assert.assertEquals(Month.AUGUST.getValue(), ldtC.getMonthValue());
		Assert.assertEquals(25, ldtC.getDayOfMonth());
		Assert.assertEquals(12, ldtC.getHour());
		Assert.assertEquals(34, ldtC.getMinute());
		Assert.assertEquals(56, ldtC.getSecond());
		Assert.assertEquals(789000000, ldtC.getNano());


		final LocalTime ltC = ConvertUtils.getDefaultInstance().cast(LocalTime.class, c);
		final LocalTime ltD = ConvertUtils.getDefaultInstance().cast(LocalTime.class, d);

		Assert.assertEquals(ltC, ltD);
		Assert.assertEquals(12, ltC.getHour());
		Assert.assertEquals(34, ltC.getMinute());
		Assert.assertEquals(56, ltC.getSecond());
		Assert.assertEquals(789000000, ltC.getNano());

		final Date convD = ConvertUtils.getDefaultInstance().cast(Date.class, ldtC);
		final Calendar convC = ConvertUtils.getDefaultInstance().cast(Calendar.class, ldtC);
		Assert.assertEquals(Calendar.AUGUST, convC.get(Calendar.MONTH));
		Assert.assertEquals(convD, convC.getTime());

		final Date convDT = ConvertUtils.getDefaultInstance().cast(Date.class, ldtC);
		final Calendar convCT = ConvertUtils.getDefaultInstance().cast(Calendar.class, ldtC);

		Assert.assertEquals(convCT, c);
		Assert.assertEquals(convDT, d);

		Year y = ConvertUtils.getDefaultInstance().cast(Year.class, "2016");
		Assert.assertEquals(Year.of(2016), y);
		y = ConvertUtils.getDefaultInstance().cast(Year.class, "2016");
		Assert.assertEquals(Year.of(2016), y);
		y = ConvertUtils.getDefaultInstance().cast(Year.class, "2016-12-25");
		Assert.assertEquals(Year.of(2016), y);

		YearMonth ym = ConvertUtils.getDefaultInstance().cast(YearMonth.class, "2016-12");
		Assert.assertEquals(YearMonth.of(2016, 12), ym);
		ym = ConvertUtils.getDefaultInstance().cast(YearMonth.class, "2016-12-25");
		Assert.assertEquals(YearMonth.of(2016, 12), ym);

		MonthDay md = ConvertUtils.getDefaultInstance().cast(MonthDay.class, "12-31");
		Assert.assertEquals(MonthDay.of(12, 31), md);
		md = ConvertUtils.getDefaultInstance().cast(MonthDay.class, "12-31");
		Assert.assertEquals(MonthDay.of(12, 31), md);

		Month m = ConvertUtils.getDefaultInstance().cast(Month.class, "09");
		Assert.assertEquals(Month.of(9), m);
		m = ConvertUtils.getDefaultInstance().cast(Month.class, "2016-09-31");
		Assert.assertEquals(Month.of(9), m);
		m = ConvertUtils.getDefaultInstance().cast(Month.class, "9");
		Assert.assertEquals(Month.of(9), m);
		m = ConvertUtils.getDefaultInstance().cast(Month.class, 9);
		Assert.assertEquals(Month.of(9), m);

		DayOfWeek day = ConvertUtils.getDefaultInstance().cast(DayOfWeek.class, "MONDAY");
		Assert.assertEquals(DayOfWeek.MONDAY, day);
		day = ConvertUtils.getDefaultInstance().cast(DayOfWeek.class, "01");
		Assert.assertEquals(DayOfWeek.MONDAY, day);
		day = ConvertUtils.getDefaultInstance().cast(DayOfWeek.class, "1");
		Assert.assertEquals(DayOfWeek.MONDAY, day);
		day = ConvertUtils.getDefaultInstance().cast(DayOfWeek.class, 1);
		Assert.assertEquals(DayOfWeek.MONDAY, day);

		Assert.assertEquals(d, ConvertUtils.getDefaultInstance().cast(Date.class, ConvertUtils.getDefaultInstance().cast(LocalDateTime.class, d.getTime())));
		Assert.assertEquals(d, ConvertUtils.getDefaultInstance().cast(Date.class, ConvertUtils.getDefaultInstance().cast(LocalDateTime.class, ConvertUtils.getDefaultInstance().cast(XMLGregorianCalendar.class, d))));
		Assert.assertEquals(ldC, ConvertUtils.getDefaultInstance().cast(LocalDate.class, ConvertUtils.getDefaultInstance().cast(LocalDateTime.class, ConvertUtils.getDefaultInstance().cast(LocalDate.class, d))));

		Assert.assertEquals("2016-08-25", ConvertUtils.getDefaultInstance().toString(d));
		Assert.assertEquals("2016-08-25", ConvertUtils.getDefaultInstance().toString(ldC));
	}

	@SuppressWarnings("unchecked")
	@Test
	public void testJdk8Var() throws Exception {
		final Jdk8VariablesManager.Jdk8Date date = new Jdk8VariablesManager.Jdk8Date();

		date.compileRead(new MCompilationContext(), Month.class);
		Assert.assertEquals(LocalDate.now().getMonth(), date.get(new MExecutionContext<>()));

		date.compileRead(new MCompilationContext(), DayOfWeek.class);
		Assert.assertEquals(LocalDate.now().getDayOfWeek(), date.get(new MExecutionContext<>()));

		date.compileRead(new MCompilationContext(), LocalDate.class);
		Assert.assertEquals(LocalDate.now(), date.get(new MExecutionContext<>()));

		date.compileRead(new MCompilationContext(), ArrayList.class, Month.class);
		final Object l = date.get(new MExecutionContext<>());
		Assert.assertEquals(ArrayList.class, l.getClass());
		Assert.assertEquals(LocalDate.now().getMonth(), ((ArrayList<?>) l).get(0));


		final Jdk8VariablesManager.Jdk8DateTime datetime = new Jdk8VariablesManager.Jdk8DateTime();
		final Jdk8VariablesManager.Jdk8Time time = new Jdk8VariablesManager.Jdk8Time();

		datetime.compileRead(new MCompilationContext(), LocalDateTime.class);
		time.compileRead(new MCompilationContext(), LocalDateTime.class);

		datetime.get(new MExecutionContext<>());
		time.get(new MExecutionContext<>());

		Assert.assertEquals(Jdk8Now.class, MAddonsFinder.getAddon("#NOW"));
	}

	@Test
	public void testDateDiff() throws Exception {
		final MValueAccessor d1 = new MValueAccessor();
		final MValueAccessor d2 = new MValueAccessor();
		final MValueAccessor f = new MValueAccessor();
		final MDateDiffAccessor diff = new MDateDiffAccessor(d1, d2, f);
		diff.compileRead(new MCompilationContext(), (Class<?>) null);
		final IExecutionContext ctxt = new MExecutionContext<>();

		d1.setValue(ConvertUtils.getDefaultInstance().cast(LocalDateTime.class, "2016-10-25 15:45:26.000"));
		d2.setValue(ConvertUtils.getDefaultInstance().cast(LocalDateTime.class, "2016-10-24 15:45:26.000"));
		f.setValue("D");
		Assert.assertEquals(1L, diff.get(ctxt));

		d2.setValue(ConvertUtils.getDefaultInstance().cast(LocalDateTime.class, "2016-10-24 15:45:26.001"));
		Assert.assertEquals(0L, diff.get(ctxt));
		d2.setValue(ConvertUtils.getDefaultInstance().cast(LocalDateTime.class, "2016-10-24 15:45:25.999"));
		Assert.assertEquals(1L, diff.get(ctxt));

		f.setValue("M");
		Assert.assertEquals(0L, diff.get(ctxt));

		d2.setValue(ConvertUtils.getDefaultInstance().cast(LocalDateTime.class, "2015-03-25 15:45:26.000"));
		Assert.assertEquals(19L, diff.get(ctxt));

		d2.setValue(ConvertUtils.getDefaultInstance().cast(LocalDateTime.class, "2015-03-25 15:45:26.001"));
		Assert.assertEquals(18L, diff.get(ctxt));

		d2.setValue(ConvertUtils.getDefaultInstance().cast(LocalDateTime.class, "2015-03-25 15:45:25.999"));
		Assert.assertEquals(19L, diff.get(ctxt));

		f.setValue("Y");
		Assert.assertEquals(1L, diff.get(ctxt));
	}

	@Test
	public void testDateMan() throws Exception {
		final MValueAccessor source = new MValueAccessor();
		final MDateManipulationAccessor acc = new MDateManipulationAccessor(
				source, new MValueAccessor(1), new MValueAccessor(-1), null, null, null, null, null);
		acc.compileRead(new MCompilationContext(), LocalDateTime.class);

		final IExecutionContext ctxt = new MExecutionContext<>();

		source.setValue("2016-01-01");
		LocalDateTime d = (LocalDateTime) acc.get(ctxt);
		Assert.assertEquals(ConvertUtils.getDefaultInstance().cast(LocalDateTime.class, "2016-12-01"), d);

		source.setValue(ConvertUtils.getDefaultInstance().cast(LocalDateTime.class, "2016-02-01"));
		d = (LocalDateTime) acc.get(ctxt);
		Assert.assertEquals(ConvertUtils.getDefaultInstance().cast(LocalDateTime.class, "2017-01-01"), d);

		acc.setDay(new MValueAccessor(-1));
		d = (LocalDateTime) acc.get(ctxt);
		Assert.assertEquals(ConvertUtils.getDefaultInstance().cast(LocalDateTime.class, "2016-12-31"), d);

		acc.setHour(new MValueAccessor(72));
		d = (LocalDateTime) acc.get(ctxt);
		Assert.assertEquals(ConvertUtils.getDefaultInstance().cast(LocalDateTime.class, "2017-01-03"), d);

		acc.setMinute(new MValueAccessor(60*24*7));
		d = (LocalDateTime) acc.get(ctxt);
		Assert.assertEquals(ConvertUtils.getDefaultInstance().cast(LocalDateTime.class, "2017-01-10"), d);

		acc.setSecond(new MValueAccessor(60*60*24*5));
		d = (LocalDateTime) acc.get(ctxt);
		Assert.assertEquals(ConvertUtils.getDefaultInstance().cast(LocalDateTime.class, "2017-01-15"), d);

		acc.setMilli(new MValueAccessor(1000*60*60*24*2));
		d = (LocalDateTime) acc.get(ctxt);
		Assert.assertEquals(ConvertUtils.getDefaultInstance().cast(LocalDateTime.class, "2017-01-17"), d);
	}

	@Test
	public void testFr() throws Exception {
		final Jdk8DateTimeConverter c = new Jdk8DateTimeConverter();
		c.setPreferredPattern("eeeee dd MMMMM yyyy");
		c.setLocale(Locale.FRANCE);

		final LocalDate d = LocalDate.of(2016, 8, 25);

		final String s = c.convertToString(d);
		Assert.assertEquals("J 25 A 2016", s);
		System.err.println(s);

		Assert.assertEquals(d, c.convertToType(LocalDate.class, s));

	}
}
